searchData={"content_type":"text/markdown","items":[{"doc":"The Device-side client for NervesHub.\n\nThe `:nerves_hub_link` Erlang application will start by default if installed\nas a dependency and use provided configuration to connect to a NervesHub\nserver.\n\nThis module primarily provides utility functions for checking the status of\nthe connection and performing some operations such as reconnecting, sending\na file to a connected console and more.","ref":"NervesHubLink.html","title":"NervesHubLink","type":"module"},{"doc":"Checks if the device is connected to the NervesHub device channel.","ref":"NervesHubLink.html#connected?/0","title":"NervesHubLink.connected?/0","type":"function"},{"doc":"Return whether there's currently an active console session","ref":"NervesHubLink.html#console_active?/0","title":"NervesHubLink.console_active?/0","type":"function"},{"doc":"Checks if the device is connected to the NervesHub console channel.","ref":"NervesHubLink.html#console_connected?/0","title":"NervesHubLink.console_connected?/0","type":"function"},{"doc":"Checks if the device is connected to the NervesHub extensions channel.","ref":"NervesHubLink.html#extensions_connected?/0","title":"NervesHubLink.extensions_connected?/0","type":"function"},{"doc":"Restart the socket and device channel","ref":"NervesHubLink.html#reconnect/0","title":"NervesHubLink.reconnect/0","type":"function"},{"doc":"Send a file to the connected console","ref":"NervesHubLink.html#send_file/1","title":"NervesHubLink.send_file/1","type":"function"},{"doc":"Send update progress percentage for display in web","ref":"NervesHubLink.html#send_update_progress/1","title":"NervesHubLink.send_update_progress/1","type":"function"},{"doc":"Send an update status to web","ref":"NervesHubLink.html#send_update_status/1","title":"NervesHubLink.send_update_status/1","type":"function"},{"doc":"Checks if the device has a socket connection with NervesHub","ref":"NervesHubLink.html#socket_connected?/0","title":"NervesHubLink.socket_connected?/0","type":"function"},{"doc":"Current status of the update manager","ref":"NervesHubLink.html#status/0","title":"NervesHubLink.status/0","type":"function"},{"doc":"Functions for computing network connect backoff intervals","ref":"NervesHubLink.Backoff.html","title":"NervesHubLink.Backoff","type":"module"},{"doc":"Produce a list of integer backoff delays with jitter\n\nThe first two parameters are minimum and maximum value. These are expected to\nbe milliseconds, but this function doesn't care. The returned list will start\nwith the minimum value and then double it until it reaches the maximum value.\n\nThe third parameter is the amount of jitter to add to each delay. The value\nshould be between 0 and 1. Zero adds no jitter. A value like 0.25 will add up\nto 25% of the delay amount.","ref":"NervesHubLink.Backoff.html#delay_list/3","title":"NervesHubLink.Backoff.delay_list/3","type":"function"},{"doc":"Certificate management utilities","ref":"NervesHubLink.Certificate.html","title":"NervesHubLink.Certificate","type":"module"},{"doc":"Returns a list of der encoded CA certs","ref":"NervesHubLink.Certificate.html#ca_certs/0","title":"NervesHubLink.Certificate.ca_certs/0","type":"function"},{"doc":"","ref":"NervesHubLink.Certificate.html#key_pem_to_der/1","title":"NervesHubLink.Certificate.key_pem_to_der/1","type":"function"},{"doc":"","ref":"NervesHubLink.Certificate.html#pem_to_der/1","title":"NervesHubLink.Certificate.pem_to_der/1","type":"function"},{"doc":"Behaviour for implementing a custom metric set to be used in a report.","ref":"NervesHubLink.Extensions.Health.MetricSet.html","title":"NervesHubLink.Extensions.Health.MetricSet","type":"behaviour"},{"doc":"","ref":"NervesHubLink.Extensions.Health.MetricSet.html#c:sample/0","title":"NervesHubLink.Extensions.Health.MetricSet.sample/0","type":"callback"},{"doc":"Health report metrics for CPU temperature, utilization, and load averages.\n\nThe keys used in the report are:\n  - cpu_temp: CPU temperature in Celsius\n  - cpu_usage_percent: CPU utilization percentage\n  - load_1min: Load average over the last minute\n  - load_5min: Load average over the last five minutes\n  - load_15min: Load average over the last fifteen minutes","ref":"NervesHubLink.Extensions.Health.MetricSet.CPU.html","title":"NervesHubLink.Extensions.Health.MetricSet.CPU","type":"module"},{"doc":"Health report metrics for total, available, and used disk space.\n\nThe keys used in the report are:\n  - disk_total_kb: Total disk space in kilobytes\n  - disk_available_kb: Available disk space in kilobytes\n  - disk_used_percentage: Used disk space percentage","ref":"NervesHubLink.Extensions.Health.MetricSet.Disk.html","title":"NervesHubLink.Extensions.Health.MetricSet.Disk","type":"module"},{"doc":"Health report metrics for total, available, and used (percentage) memory.\n\nThe keys used in the report are:\n  - mem_size_mb: Total memory size in megabytes\n  - mem_used_mb: Used memory size in megabytes\n  - mem_used_percent: Used memory percentage","ref":"NervesHubLink.Extensions.Health.MetricSet.Memory.html","title":"NervesHubLink.Extensions.Health.MetricSet.Memory","type":"module"},{"doc":"Health report metrics for bytes sent and received (total) per interface.\n\nThe keys used in the report are:\n  - [interface]_bytes_received_total: Total bytes received by the interface\n  - [interface]_bytes_sent_total: Total bytes sent by the interface","ref":"NervesHubLink.Extensions.Health.MetricSet.NetworkTraffic.html","title":"NervesHubLink.Extensions.Health.MetricSet.NetworkTraffic","type":"module"},{"doc":"A behaviour module for customizing if and when firmware updates get applied.\n\nBy default NervesHubLink applies updates as soon as it knows about them from the\nNervesHubLink server and doesn't give warning before rebooting. This let's\ndevices hook into the decision making process and monitor the update's\nprogress.\n\n# Example\n\n```elixir\ndefmodule MyApp.NervesHubLinkClient do\n  @behaviour NervesHubLink.Client\n\n  # May return:\n  #  * `:apply` - apply the action immediately\n  #  * `:ignore` - don't apply the action, don't ask again.\n  #  * `{:reschedule, timeout_in_milliseconds}` - call this function again later.\n\n  @impl NervesHubLink.Client\n  def update_available(data) do\n    if SomeInternalAPI.is_now_a_good_time_to_update?(data) do\n      :apply\n    else\n      {:reschedule, 60_000}\n    end\n  end\nend\n```\n\nTo have NervesHubLink invoke it, add the following to your `config.exs`:\n\n```elixir\nconfig :nerves_hub_link, client: MyApp.NervesHubLinkClient\n```","ref":"NervesHubLink.Client.html","title":"NervesHubLink.Client","type":"behaviour"},{"doc":"","ref":"NervesHubLink.Client.html#archive_available/1","title":"NervesHubLink.Client.archive_available/1","type":"function"},{"doc":"Called when an archive is available for download\n\nMay return one of:\n\n* `download` - Download the archive right now\n* `ignore` - Don't download this archive.\n* `{:reschedule, timeout}` - Defer making a decision. Call this function again in `timeout` milliseconds.","ref":"NervesHubLink.Client.html#c:archive_available/1","title":"NervesHubLink.Client.archive_available/1","type":"callback"},{"doc":"Archive that comes over a socket.","ref":"NervesHubLink.Client.html#t:archive_data/0","title":"NervesHubLink.Client.archive_data/0","type":"type"},{"doc":"","ref":"NervesHubLink.Client.html#archive_ready/2","title":"NervesHubLink.Client.archive_ready/2","type":"function"},{"doc":"Called when an archive has been downloaded and is available for the application to do something","ref":"NervesHubLink.Client.html#c:archive_ready/2","title":"NervesHubLink.Client.archive_ready/2","type":"callback"},{"doc":"Supported responses from `archive_available/1`","ref":"NervesHubLink.Client.html#t:archive_response/0","title":"NervesHubLink.Client.archive_response/0","type":"type"},{"doc":"Firmware update progress, completion or error report","ref":"NervesHubLink.Client.html#t:fwup_message/0","title":"NervesHubLink.Client.fwup_message/0","type":"type"},{"doc":"This function is called internally by NervesHubLink to notify clients of fwup errors.","ref":"NervesHubLink.Client.html#handle_error/1","title":"NervesHubLink.Client.handle_error/1","type":"function"},{"doc":"Called when downloading a firmware update fails.\n\nThe return value of this function is not checked.","ref":"NervesHubLink.Client.html#c:handle_error/1","title":"NervesHubLink.Client.handle_error/1","type":"callback"},{"doc":"This function is called internally by NervesHubLink to notify clients of fwup progress.","ref":"NervesHubLink.Client.html#handle_fwup_message/1","title":"NervesHubLink.Client.handle_fwup_message/1","type":"function"},{"doc":"Called on firmware update reports.\n\nThe return value of this function is not checked.","ref":"NervesHubLink.Client.html#c:handle_fwup_message/1","title":"NervesHubLink.Client.handle_fwup_message/1","type":"callback"},{"doc":"This function is called internally by NervesHubLink to identify a device.","ref":"NervesHubLink.Client.html#identify/0","title":"NervesHubLink.Client.identify/0","type":"function"},{"doc":"Callback to identify the device from NervesHub.","ref":"NervesHubLink.Client.html#c:identify/0","title":"NervesHubLink.Client.identify/0","type":"callback"},{"doc":"This function is called internally by NervesHubLink to initiate a reboot.\n\nAfter a successful firmware update, NervesHubLink calls this to start the\nreboot process. It calls `c:reboot/0` if supplied or\n`Nerves.Runtime.reboot/0`.","ref":"NervesHubLink.Client.html#initiate_reboot/0","title":"NervesHubLink.Client.initiate_reboot/0","type":"function"},{"doc":"Optional callback to reboot the device when a firmware update completes\n\nThe default behavior is to call `Nerves.Runtime.reboot/0` after a successful update. This\nis useful for testing and for doing additional work like notifying users in a UI that a reboot\nwill happen soon. It is critical that a reboot does happen.","ref":"NervesHubLink.Client.html#c:reboot/0","title":"NervesHubLink.Client.reboot/0","type":"callback"},{"doc":"This function is called internally by NervesHubLink to notify clients of disconnects.","ref":"NervesHubLink.Client.html#reconnect_backoff/0","title":"NervesHubLink.Client.reconnect_backoff/0","type":"function"},{"doc":"Optional callback when the socket disconnected, before starting to reconnect.\n\nThe return value is used to reset the next socket's retry timeout. `nil` asks NervesHubLink\nto calculate a set of random backoffs to use.\n\nYou may wish to use this to dynamically change the reconnect backoffs. For instance,\nduring a NervesHub deploy you may wish to change the reconnect based on your\nown logic to not create a thundering herd of reconnections. If you have a particularly\nflaky connection you can increase how fast the reconnect happens to avoid overloading\nyour server.","ref":"NervesHubLink.Client.html#c:reconnect_backoff/0","title":"NervesHubLink.Client.reconnect_backoff/0","type":"callback"},{"doc":"This function is called internally by NervesHubLink to notify clients.","ref":"NervesHubLink.Client.html#update_available/1","title":"NervesHubLink.Client.update_available/1","type":"function"},{"doc":"Called to find out what to do when a firmware update is available.\n\nMay return one of:\n\n* `apply` - Download and apply the update right now.\n* `ignore` - Don't download and apply this update.\n* `{:reschedule, timeout}` - Defer making a decision. Call this function again in `timeout` milliseconds.","ref":"NervesHubLink.Client.html#c:update_available/1","title":"NervesHubLink.Client.update_available/1","type":"callback"},{"doc":"Update that comes over a socket.","ref":"NervesHubLink.Client.html#t:update_data/0","title":"NervesHubLink.Client.update_data/0","type":"type"},{"doc":"Supported responses from `update_available/1`","ref":"NervesHubLink.Client.html#t:update_response/0","title":"NervesHubLink.Client.update_response/0","type":"type"},{"doc":"Default NervesHubLink.Client implementation\n\nThis client always accepts an update.","ref":"NervesHubLink.Client.Default.html","title":"NervesHubLink.Client.Default","type":"module"},{"doc":"Behaviour for implementing a configurator.\n\nImplementing a new one would typically be relevant if you need a new way\nof doing authentication or other configuration. New security hardware or\na new way to make the private key, shared secret available.\n\nIf you want to control the firmware updates look at `NervesHubLink.Client`\ninstead.","ref":"NervesHubLink.Configurator.html","title":"NervesHubLink.Configurator","type":"behaviour"},{"doc":"","ref":"NervesHubLink.Configurator.html#build/0","title":"NervesHubLink.Configurator.build/0","type":"function"},{"doc":"","ref":"NervesHubLink.Configurator.html#c:build/1","title":"NervesHubLink.Configurator.build/1","type":"callback"},{"doc":"","ref":"NervesHubLink.Configurator.html#fetch_configurator/0","title":"NervesHubLink.Configurator.fetch_configurator/0","type":"function"},{"doc":"Data structure for holding configuration information provided by the configurator.","ref":"NervesHubLink.Configurator.Config.html","title":"NervesHubLink.Configurator.Config","type":"module"},{"doc":"","ref":"NervesHubLink.Configurator.Config.html#t:t/0","title":"NervesHubLink.Configurator.Config.t/0","type":"type"},{"doc":"Configurator allowing authentication via locally stored certificate key.","ref":"NervesHubLink.Configurator.LocalCertKey.html","title":"NervesHubLink.Configurator.LocalCertKey","type":"module"},{"doc":"Configurator enabling authentication via NervesKey.","ref":"NervesHubLink.Configurator.NervesKey.html","title":"NervesHubLink.Configurator.NervesKey","type":"module"},{"doc":"Configurator allowing authentication with a shared secret.","ref":"NervesHubLink.Configurator.SharedSecret.html","title":"NervesHubLink.Configurator.SharedSecret","type":"module"},{"doc":"Generate headers for Shared Secret Auth","ref":"NervesHubLink.Configurator.SharedSecret.html#headers/1","title":"NervesHubLink.Configurator.SharedSecret.headers/1","type":"function"},{"doc":"Config structure responsible for:\n\n- handling callbacks from FWUP\n- applying a fwupdate,\n- and storing fwup task configuration","ref":"NervesHubLink.FwupConfig.html","title":"NervesHubLink.FwupConfig","type":"module"},{"doc":"`handle_fwup_message` will be called with this data","ref":"NervesHubLink.FwupConfig.html#t:fwup_message/0","title":"NervesHubLink.FwupConfig.fwup_message/0","type":"type"},{"doc":"Callback that will be called during the lifecycle of a fwupdate being applied","ref":"NervesHubLink.FwupConfig.html#t:handle_fwup_message_fun/0","title":"NervesHubLink.FwupConfig.handle_fwup_message_fun/0","type":"type"},{"doc":"","ref":"NervesHubLink.FwupConfig.html#t:t/0","title":"NervesHubLink.FwupConfig.t/0","type":"type"},{"doc":"Called when an update has been dispatched via `NervesHubLink.UpdateManager.apply_update/2`","ref":"NervesHubLink.FwupConfig.html#t:update_available_fun/0","title":"NervesHubLink.FwupConfig.update_available_fun/0","type":"type"},{"doc":"Raises an ArgumentError on invalid arguments","ref":"NervesHubLink.FwupConfig.html#validate!/1","title":"NervesHubLink.FwupConfig.validate!/1","type":"function"},{"doc":"Extensions are a mechanism for transmitting messages for non-critical\nfunctionality over the existing NervesHub Socket. An extension will only\nattach if the server-side requests it from the device to ensure it will not\ndisrupt regular operation.\n\nThis module provides a behaviour with a macro to use for implementing an\nExtension.\n\nExtensions are started as separate GenServers under a DynamicSupervisor and\nany messages namespaced for a specific extension will be forwarded to that\nextension's GenServer.","ref":"NervesHubLink.Extensions.html","title":"NervesHubLink.Extensions","type":"behaviour"},{"doc":"Attach specified extensions","ref":"NervesHubLink.Extensions.html#attach/1","title":"NervesHubLink.Extensions.attach/1","type":"function"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"NervesHubLink.Extensions.html#child_spec/1","title":"NervesHubLink.Extensions.child_spec/1","type":"function"},{"doc":"Detach specified extensions\n\nAlso supports `:all` as an argument for cases NervesHubLink\nmay want to detach all of them at once","ref":"NervesHubLink.Extensions.html#detach/1","title":"NervesHubLink.Extensions.detach/1","type":"function"},{"doc":"","ref":"NervesHubLink.Extensions.html#handle_event/2","title":"NervesHubLink.Extensions.handle_event/2","type":"function"},{"doc":"Invoked when routing an Extension event\n\nBehaves the same as `c:GenServer.handle_info/2`","ref":"NervesHubLink.Extensions.html#c:handle_event/3","title":"NervesHubLink.Extensions.handle_event/3","type":"callback"},{"doc":"List extensions currently available","ref":"NervesHubLink.Extensions.html#list/0","title":"NervesHubLink.Extensions.list/0","type":"function"},{"doc":"","ref":"NervesHubLink.Extensions.html#start_link/1","title":"NervesHubLink.Extensions.start_link/1","type":"function"},{"doc":"The Geo Extension.\n\nProvides geo-location information to NervesHub.","ref":"NervesHubLink.Extensions.Geo.html","title":"NervesHubLink.Extensions.Geo","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"NervesHubLink.Extensions.Geo.html#child_spec/1","title":"NervesHubLink.Extensions.Geo.child_spec/1","type":"function"},{"doc":"","ref":"NervesHubLink.Extensions.Geo.html#push/2","title":"NervesHubLink.Extensions.Geo.push/2","type":"function"},{"doc":"","ref":"NervesHubLink.Extensions.Geo.html#start_link/1","title":"NervesHubLink.Extensions.Geo.start_link/1","type":"function"},{"doc":"Default `Resolver` implementation.\n\nUses the Nerves project's `whenwhere` service and library to perform a rough\nGeoIP-lookup. Please use within reason. The Nerves team provides no\nguarantees for this service's availability or continued operation. With some\nluck it should be up, reliable and useful.","ref":"NervesHubLink.Extensions.Geo.DefaultResolver.html","title":"NervesHubLink.Extensions.Geo.DefaultResolver","type":"module"},{"doc":"Geo extension behaviour for writing custom resolvers.\n\nFor example to support your GPS or LTE modem's means of geo-location.\n\nDefault implementation is `NervesHubLink.Extensions.Geo.DefaultResolver`.","ref":"NervesHubLink.Extensions.Geo.Resolver.html","title":"NervesHubLink.Extensions.Geo.Resolver","type":"behaviour"},{"doc":"Formatted error response from a failed location resolution","ref":"NervesHubLink.Extensions.Geo.Resolver.html#t:error_information/0","title":"NervesHubLink.Extensions.Geo.Resolver.error_information/0","type":"type"},{"doc":"Location information from a successful location resolution","ref":"NervesHubLink.Extensions.Geo.Resolver.html#t:location_information/0","title":"NervesHubLink.Extensions.Geo.Resolver.location_information/0","type":"type"},{"doc":"Supported responses from `resolve_location/0`","ref":"NervesHubLink.Extensions.Geo.Resolver.html#t:location_responses/0","title":"NervesHubLink.Extensions.Geo.Resolver.location_responses/0","type":"type"},{"doc":"","ref":"NervesHubLink.Extensions.Geo.Resolver.html#c:resolve_location/0","title":"NervesHubLink.Extensions.Geo.Resolver.resolve_location/0","type":"callback"},{"doc":"The Health Extension.\n\nProvides metrics, metadata and alarms to allow building an understanding of\nthe operational state of a device. The device's \"health\". This information\nis reported over the extensions mechanism to NervesHub for display, alerting\nand more.","ref":"NervesHubLink.Extensions.Health.html","title":"NervesHubLink.Extensions.Health","type":"module"},{"doc":"","ref":"NervesHubLink.Extensions.Health.html#check_health/1","title":"NervesHubLink.Extensions.Health.check_health/1","type":"function"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"NervesHubLink.Extensions.Health.html#child_spec/1","title":"NervesHubLink.Extensions.Health.child_spec/1","type":"function"},{"doc":"","ref":"NervesHubLink.Extensions.Health.html#push/2","title":"NervesHubLink.Extensions.Health.push/2","type":"function"},{"doc":"Confirms if a health report has been sent.","ref":"NervesHubLink.Extensions.Health.html#report_sent?/0","title":"NervesHubLink.Extensions.Health.report_sent?/0","type":"function"},{"doc":"iex> NervesHubLink.Extensions.Health.report_sent?()\n    false\n\n    iex> NervesHubLink.Extensions.Health.send_report()\n    :ok\n\n    iex> NervesHubLink.Extensions.Health.report_sent?()\n    true","ref":"NervesHubLink.Extensions.Health.html#report_sent?/0-examples","title":"Examples - NervesHubLink.Extensions.Health.report_sent?/0","type":"function"},{"doc":"Request a health report be sent asynchronously.","ref":"NervesHubLink.Extensions.Health.html#send_report/0","title":"NervesHubLink.Extensions.Health.send_report/0","type":"function"},{"doc":"iex> NervesHubLink.Extensions.Health.send_report()\n    :ok","ref":"NervesHubLink.Extensions.Health.html#send_report/0-examples","title":"Examples - NervesHubLink.Extensions.Health.send_report/0","type":"function"},{"doc":"","ref":"NervesHubLink.Extensions.Health.html#start_link/1","title":"NervesHubLink.Extensions.Health.start_link/1","type":"function"},{"doc":"A default health report implementation with support for easily adding\nnew metadata, metrics and such via config.","ref":"NervesHubLink.Extensions.Health.DefaultReport.html","title":"NervesHubLink.Extensions.Health.DefaultReport","type":"module"},{"doc":"Structure for device status.","ref":"NervesHubLink.Extensions.Health.DeviceStatus.html","title":"NervesHubLink.Extensions.Health.DeviceStatus","type":"module"},{"doc":"","ref":"NervesHubLink.Extensions.Health.DeviceStatus.html#t:alarm_description/0","title":"NervesHubLink.Extensions.Health.DeviceStatus.alarm_description/0","type":"type"},{"doc":"","ref":"NervesHubLink.Extensions.Health.DeviceStatus.html#t:alarm_id/0","title":"NervesHubLink.Extensions.Health.DeviceStatus.alarm_id/0","type":"type"},{"doc":"","ref":"NervesHubLink.Extensions.Health.DeviceStatus.html#t:connection_status/0","title":"NervesHubLink.Extensions.Health.DeviceStatus.connection_status/0","type":"type"},{"doc":"","ref":"NervesHubLink.Extensions.Health.DeviceStatus.html#t:interface_identifer/0","title":"NervesHubLink.Extensions.Health.DeviceStatus.interface_identifer/0","type":"type"},{"doc":"","ref":"NervesHubLink.Extensions.Health.DeviceStatus.html#t:interface_type/0","title":"NervesHubLink.Extensions.Health.DeviceStatus.interface_type/0","type":"type"},{"doc":"","ref":"NervesHubLink.Extensions.Health.DeviceStatus.html#new/1","title":"NervesHubLink.Extensions.Health.DeviceStatus.new/1","type":"function"},{"doc":"","ref":"NervesHubLink.Extensions.Health.DeviceStatus.html#t:t/0","title":"NervesHubLink.Extensions.Health.DeviceStatus.t/0","type":"type"},{"doc":"Behaviour for implementing a custom health report.\n\nThe `NervesHubLink.Extensions.Health.DefaultReport` has a lot of easy\ncustomization options available. If you want an entirely custom report or\nneed exact control over how the generation of data happens then using this\ngives you that possibility.","ref":"NervesHubLink.Extensions.Health.Report.html","title":"NervesHubLink.Extensions.Health.Report","type":"behaviour"},{"doc":"","ref":"NervesHubLink.Extensions.Health.Report.html#c:alarms/0","title":"NervesHubLink.Extensions.Health.Report.alarms/0","type":"callback"},{"doc":"","ref":"NervesHubLink.Extensions.Health.Report.html#c:checks/0","title":"NervesHubLink.Extensions.Health.Report.checks/0","type":"callback"},{"doc":"","ref":"NervesHubLink.Extensions.Health.Report.html#c:connectivity/0","title":"NervesHubLink.Extensions.Health.Report.connectivity/0","type":"callback"},{"doc":"","ref":"NervesHubLink.Extensions.Health.Report.html#c:metadata/0","title":"NervesHubLink.Extensions.Health.Report.metadata/0","type":"callback"},{"doc":"","ref":"NervesHubLink.Extensions.Health.Report.html#c:metrics/0","title":"NervesHubLink.Extensions.Health.Report.metrics/0","type":"callback"},{"doc":"","ref":"NervesHubLink.Extensions.Health.Report.html#c:timestamp/0","title":"NervesHubLink.Extensions.Health.Report.timestamp/0","type":"callback"},{"doc":"GenServer for handling downloading archives from NervesHub\n\nYour NervesHubLink client will tell the manager when to download\nan archive and the Manager will tell your client when it's done\ndownloading so you can act on it.\n\nYou are expected to remove the file when you're done with it and track\nthat it has been applied to prevent downloading again.","ref":"NervesHubLink.ArchiveManager.html","title":"NervesHubLink.ArchiveManager","type":"module"},{"doc":"Must be called when an archive payload is dispatched from\nNervesHub. the map must contain a `\"url\"` key.","ref":"NervesHubLink.ArchiveManager.html#apply_archive/3","title":"NervesHubLink.ArchiveManager.apply_archive/3","type":"function"},{"doc":"Returns the UUID of the currently downloading archive, or nil.","ref":"NervesHubLink.ArchiveManager.html#currently_downloading_uuid/1","title":"NervesHubLink.ArchiveManager.currently_downloading_uuid/1","type":"function"},{"doc":"","ref":"NervesHubLink.ArchiveManager.html#t:status/0","title":"NervesHubLink.ArchiveManager.status/0","type":"type"},{"doc":"Returns the current status of the archive manager","ref":"NervesHubLink.ArchiveManager.html#status/1","title":"NervesHubLink.ArchiveManager.status/1","type":"function"},{"doc":"Handles downloading files via HTTP.\n\nSeveral interesting properties about the download are internally cached, such as:\n\n  * the URI of the request\n  * the total content amounts of bytes of the file being downloaded\n  * the total amount of bytes downloaded at any given time\n\nUsing this information, it can restart a download using the\n[`Range` HTTP header](https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Range).\n\nThis process's **only** focus is obtaining data reliably. It doesn't have any\nside effects on the system.\n\nYou can configure various options related to how the `Downloader` handles timeouts,\ndisconnections, and other aspects of the retry logic by adding the following configuration\nto your application's config file:\n\n    config :nerves_hub_link, :retry_config,\n      max_disconnects: 20,\n      idle_timeout: 75_000,\n      max_timeout: 10_800_000\n\nFor more information about the configuration options, see the [RetryConfig](`NervesHubLink.Downloader.RetryConfig`) module.","ref":"NervesHubLink.Downloader.html","title":"NervesHubLink.Downloader","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"NervesHubLink.Downloader.html#child_spec/1","title":"NervesHubLink.Downloader.child_spec/1","type":"function"},{"doc":"","ref":"NervesHubLink.Downloader.html#t:event_handler_fun/0","title":"NervesHubLink.Downloader.event_handler_fun/0","type":"type"},{"doc":"","ref":"NervesHubLink.Downloader.html#t:handler_event/0","title":"NervesHubLink.Downloader.handler_event/0","type":"type"},{"doc":"","ref":"NervesHubLink.Downloader.html#t:initialized_download/0","title":"NervesHubLink.Downloader.initialized_download/0","type":"type"},{"doc":"","ref":"NervesHubLink.Downloader.html#t:resume_rescheduled/0","title":"NervesHubLink.Downloader.resume_rescheduled/0","type":"type"},{"doc":"","ref":"NervesHubLink.Downloader.html#t:retry_args/0","title":"NervesHubLink.Downloader.retry_args/0","type":"type"},{"doc":"Begins downloading a file at `url` handled by `fun`.\n\n# Example\n\n      iex> pid = self()\n      #PID<0.110.0>\n      iex> fun = fn {:data, data} -> File.write(\"index.html\", data)\n      ...> {:error, error} -> IO.puts(\"error streaming file: #{inspect(error)}\")\n      ...> :complete -> send pid, :complete\n      ...> end\n      #Function<44.97283095/1 in :erl_eval.expr/5>\n      iex> NervesHubLink.Downloader.start_download(\"https://httpbin.com/\", fun)\n      {:ok, #PID<0.111.0>}\n      iex> flush()\n      :complete","ref":"NervesHubLink.Downloader.html#start_download/2","title":"NervesHubLink.Downloader.start_download/2","type":"function"},{"doc":"","ref":"NervesHubLink.Downloader.html#start_download/3","title":"NervesHubLink.Downloader.start_download/3","type":"function"},{"doc":"","ref":"NervesHubLink.Downloader.html#t:t/0","title":"NervesHubLink.Downloader.t/0","type":"type"},{"doc":"Download retry configuration.\n\nThis module provides configuration for how the `Downloader` process will\nhandle disconnects, errors, and timeouts.","ref":"NervesHubLink.Downloader.RetryConfig.html","title":"NervesHubLink.Downloader.RetryConfig","type":"module"},{"doc":"* `:max_disconnects` (`t:non_neg_integer/0`) - Maximum number of disconnects.\n  After this limit is reached the download will be stopped and will no longer be retried. The default value is `10`.\n\n* `:idle_timeout` (`t:non_neg_integer/0`) - Time (in milliseconds) between chunks of data received that once elapsed will trigger a retry.\n  This event counts towards the `max_disconnects` counter. The default value is `60000`.\n\n* `:max_timeout` (`t:non_neg_integer/0`) - Maximum time (in milliseconds) that a download can exist for.\n  After this amount of time has elapsed, the download is canceled and the download process will crash.\n  The default value is 24 hours. The default value is `86400000`.\n\n* `:time_between_retries` (`t:non_neg_integer/0`) - Time (in milliseconds) to wait before attempting to retry a download. The default value is `15000`.\n\n* `:worst_case_download_speed` (`t:non_neg_integer/0`) - Worst case download speed specified in bytes per second.\n  This is used to calculate the \"worst case\" (\"sensible\") download timeout and is\n  intended to fail faster than waiting for `max_timeout` to elapse.\n  For reference, LTE Cat M1 modems sometimes top out at 32 kbps (30 kbps for some slack). The default value is `30000`.","ref":"NervesHubLink.Downloader.RetryConfig.html#module-options","title":"Options - NervesHubLink.Downloader.RetryConfig","type":"module"},{"doc":"","ref":"NervesHubLink.Downloader.RetryConfig.html#t:t/0","title":"NervesHubLink.Downloader.RetryConfig.t/0","type":"type"},{"doc":"Validates a proposed configuration, returning the default configuration on error","ref":"NervesHubLink.Downloader.RetryConfig.html#validate/1","title":"NervesHubLink.Downloader.RetryConfig.validate/1","type":"function"},{"doc":"GenServer responsible for brokering messages between:\n  * an external controlling process\n  * FWUP\n  * HTTP\n\nShould be started in a supervision tree","ref":"NervesHubLink.UpdateManager.html","title":"NervesHubLink.UpdateManager","type":"module"},{"doc":"Must be called when an update payload is dispatched from\nNervesHub. the map must contain a `\"firmware_url\"` key.","ref":"NervesHubLink.UpdateManager.html#apply_update/3","title":"NervesHubLink.UpdateManager.apply_update/3","type":"function"},{"doc":"Returns the UUID of the currently downloading firmware, or nil.","ref":"NervesHubLink.UpdateManager.html#currently_downloading_uuid/1","title":"NervesHubLink.UpdateManager.currently_downloading_uuid/1","type":"function"},{"doc":"","ref":"NervesHubLink.UpdateManager.html#t:status/0","title":"NervesHubLink.UpdateManager.status/0","type":"type"},{"doc":"Returns the current status of the update manager","ref":"NervesHubLink.UpdateManager.html#status/1","title":"NervesHubLink.UpdateManager.status/1","type":"function"},{"doc":"Payload received from NervesHub when an archive is available.","ref":"NervesHubLink.Message.ArchiveInfo.html","title":"NervesHubLink.Message.ArchiveInfo","type":"module"},{"doc":"Parse an update message from NervesHub.","ref":"NervesHubLink.Message.ArchiveInfo.html#parse/1","title":"NervesHubLink.Message.ArchiveInfo.parse/1","type":"function"},{"doc":"","ref":"NervesHubLink.Message.ArchiveInfo.html#t:t/0","title":"NervesHubLink.Message.ArchiveInfo.t/0","type":"type"},{"doc":"Detailed firmware metadata received during the firmware update process.","ref":"NervesHubLink.Message.FirmwareMetadata.html","title":"NervesHubLink.Message.FirmwareMetadata","type":"module"},{"doc":"","ref":"NervesHubLink.Message.FirmwareMetadata.html#parse/1","title":"NervesHubLink.Message.FirmwareMetadata.parse/1","type":"function"},{"doc":"","ref":"NervesHubLink.Message.FirmwareMetadata.html#t:t/0","title":"NervesHubLink.Message.FirmwareMetadata.t/0","type":"type"},{"doc":"Payload received from NervesHub when an update is available.","ref":"NervesHubLink.Message.UpdateInfo.html","title":"NervesHubLink.Message.UpdateInfo","type":"module"},{"doc":"Parse an update message from NervesHub.","ref":"NervesHubLink.Message.UpdateInfo.html#parse/1","title":"NervesHubLink.Message.UpdateInfo.parse/1","type":"function"},{"doc":"","ref":"NervesHubLink.Message.UpdateInfo.html#t:t/0","title":"NervesHubLink.Message.UpdateInfo.t/0","type":"type"},{"doc":"# NervesHubLink\n\n[![Hex version](https://img.shields.io/hexpm/v/nerves_hub_link.svg \"Hex version\")](https://hex.pm/packages/nerves_hub_link)\n[![API docs](https://img.shields.io/hexpm/v/nerves_hub_link.svg?label=hexdocs \"API docs\")](https://hexdocs.pm/nerves_hub_link/NervesHubLink.html)\n[![CircleCI](https://dl.circleci.com/status-badge/img/gh/nerves-hub/nerves_hub_link/tree/main.svg?style=svg)](https://dl.circleci.com/status-badge/redirect/gh/nerves-hub/nerves_hub_link/tree/main)\n[![REUSE status](https://api.reuse.software/badge/github.com/nerves-hub/nerves_hub_link)](https://api.reuse.software/info/github.com/nerves-hub/nerves_hub_link)\n\n`NervesHubLink` is the supported client library for connecting devices to [NervesHub](https://github.com/nerves-hub/nerves_hub_web).\n\nThis integration includes out-of-the-box support for:\n\n- firmware updates\n- debug tooling (iex console and support scripts)\n- and device health telemetry","ref":"readme.html","title":"NervesHubLink","type":"extras"},{"doc":"[NervesHub](https://www.nerves-hub.org/) is an open-source IoT fleet management platform that is built specifically for Nerves-based devices.\n\nDevices connect to the server by joining a long-lived websocket channel.\n\nThe NervesHub platform helps schedule firmware updates, providing firmware download information to the device which can be applied\nimmediately or [when convenient](guides/configuration.md#conditionally-applying-updates).\n\nNervesHub does impose some requirements on devices and firmware that may require changes to your Nerves projects:\n\n- Firmware images are cryptographically signed (both NervesHub and devices validate signatures)\n- Devices are identified by a unique serial number\n\nWhen using client certificate authentication, each device will also require its own SSL certificate for authentication with NervesHub.\n\nThese changes enable NervesHub to provide assurances that the firmware you intend to install on a set of devices make it to those devices unaltered.\n\n> #### Info {: .info}\n>\n> This is the 2.x version of `NervesHubLink`.\n>\n> If you have been using [NervesHub](https://github.com/nerves-hub/nerves_hub_web) prior to around April, 2023 and have not updated to 2.0 or newer, please checkout the `maint-v1` branch.","ref":"readme.html#overview","title":"Overview - NervesHubLink","type":"extras"},{"doc":"The following sections will walk you through updating your Nerves project to work with a NervesHub server.\n\nMany of the steps below can be automated by NervesHub users to set up automatic firmware updates from CI and to manufacture large numbers of devices.","ref":"readme.html#getting-started","title":"Getting started - NervesHubLink","type":"extras"},{"doc":"The first step is to add `nerves_hub_link` to your target dependencies in your\nproject's `mix.exs`. For example:\n\n```elixir\n  defp deps(target) do\n    [\n      {:nerves_runtime, \"~> 0.13\"},\n      {:nerves_hub_link, \"~> 2.2\"},\n      ...\n    ] ++ system(target)\n  end\n```","ref":"readme.html#adding-nerveshublink-to-your-project","title":"Adding NervesHubLink to your project - NervesHubLink","type":"extras"},{"doc":"#### Shared secret device authentication\n\n_Important: Shared Secret authentication tailored for Hobby and R&D projects._\n\nShared Secrets use [HMAC](https://en.wikipedia.org/wiki/HMAC) cryptography to generate an authentication token used during websocket connection.\n\nThis has been built with simple device registration in mind, an ideal fit for hobby projects or projects under early R&D.\n\nYou can generate a key and secret in your NervesHub Product settings which you then include in your `NervesHubLink` settings.\n\nA full example config:\n\n```elixir\nconfig :nerves_hub_link,\n  host: \"your.nerveshub.host\",\n  shared_secret: [\n    product_key: \" \",\n    product_secret: \" \",\n  ]\n```\n\n#### NervesKey (with cert based auth)\n\n_Important: This is recommended for production device fleets._\n\nIf your project is using [NervesKey](https://github.com/nerves-hub/nerves_key), you can tell `NervesHubLink` to read those certificates and key from the chip and assign the SSL options for you by adding it as a dependency:\n\n```elixir\ndef deps() do\n  [\n    {:nerves_key, \"~> 1.2\"}\n  ]\nend\n```\n\nThis allows your config to be simplified to:\n\n```elixir\nconfig :nerves_hub_link,\n  host: \"your.nerveshub.host\"\n```\n\nNervesKey will default to using I2C bus 1 and the `:primary` certificate pair (`:primary` is one-time configurable and `:aux` may be updated). You can customize these options to use a different bus and certificate pair:\n\n```elixir\nconfig :nerves_hub_link, :nerves_key,\n  certificate_pair: :aux,\n  i2c_bus: 0\n```\n\n#### Certificate device authentication\n\nIf you would like to use certificate device authentication, but you are not using `NervesKey`, you can tell `NervesHubLink` to read the certificate and key from the file system by using:\n\n```elixir\nconfig :nerves_hub_link,\n  host: \"your.nerveshub.host\",\n  configurator: NervesHubLink.Configurator.LocalCertKey\n```\n\nBy default the configurator will use a certificate found at `/data/nerves_hub/cert.pem` and a key found at `/data/nerves_hub/key.pem`. If these are stored somewhere differently then you can specify `certfile` and `keyfile` in the `ssl` config, e.g.:\n\n```elixir\nconfig :nerves_hub_link,\n  host: \"your.nerveshub.host\",\n  configurator: NervesHubLink.Configurator.LocalCertKey,\n  ssl: [\n    certfile: \"/path/to/certfile.pem\",\n    keyfile: \"/path/to/keyfile.key\"\n  ]\n```\n\nFor more information on how to generate device certificates, please read the [\"Initializing devices\"](#https://github.com/nerves-hub/nerves_hub_cli#initializing-devices) section in the `NervesHubCLI` readme.\n\n#### Additional notes\n\nAny [valid Erlang ssl socket option](http://erlang.org/doc/man/ssl.html#TLS/DTLS%20OPTION%20DESCRIPTIONS%20-%20COMMON%20for%20SERVER%20and%20CLIENT) can go in the `:ssl` key. These options are passed to [Mint](https://hex.pm/packages/mint) by [Slipstream](https://hex.pm/packages/slipstream), which `NervesHubLink` uses for websocket connections.","ref":"readme.html#connecting-your-device-to-nerveshub","title":"Connecting your device to NervesHub - NervesHubLink","type":"extras"},{"doc":"- [Configuration](guides/configuration.md): Runtime and compile time configuration examples.\n- [Extensions](guides/extensions.md): Setup and configure Health and Geo extensions.\n- [Debugging](guides/debugging.md): Tips to debug connection issues.","ref":"readme.html#additional-guides","title":"Additional guides - NervesHubLink","type":"extras"},{"doc":"`NervesHubLink.Configurator` includes two internal versions for NervesHub to determine what extensions are available on the device socket.\n\n### `device_api_version`\n\n- `2.2.0` - Report what extensions are enabled and their version\n- `2.1.0` - Run scripts on a device separate from the console, sync firmware keys and archive keys\n- `2.0.0` - Identify a device, archives\n- `1.0.0` - Updating firmware, status updates, reboot device\n\n### `console_version`\n\n- `2.0.0` - Send and receive files from a device\n- `1.0.0` - Remote IEx console","ref":"readme.html#internal-api-versions","title":"Internal API Versions - NervesHubLink","type":"extras"},{"doc":"# Changelog\n\n## [2.7.2] - 2025-03-28\n\n* Fixed\n  * Retrieve disk info differently based on OTP version\n  * Emit a warning when trying to retrieve public certs through Erlang on   1.0`\n\n## [1.1.0] - 2021-12-09\n\n* Removed\n  * Elixir 1.8 and 1.9 are no longer supported. These hadn't been tested on CI\n    and were removed when we started testing Elixir 1.13.\n\n* Added\n  * Retry timeouts are now being set on Slipstream via the\n    `:reconnect_after_msec` parameter.  The timeouts start at 1 second and\n    double up to 60 seconds plus a random amount of jitter. Previously the\n    timeouts started under a second and maxed out at 5 seconds without jitter.\n    These timeouts were chosen to reduce the load on NervesHub servers when\n    large numbers of devices disconnect. They can be overridden.\n\n## [1.0.1] - 2021-11-16\n\n* Fixed\n  * A crash in the remote console would occur if a window resize message was received\n    before the IEx process was started\n\n## [1.0.0] - 2021-10-25\n\nThis release only bumps the version number. It doesn't have any code changes.\n\n## [0.13.1] - 2021-10-19\n\n* Fixed\n  * `NervesHubLink.reconnect/0` no longer times out and instead disconnects\n    the socket forcing the reconnection logic\n\n## [0.13.0] - 2021-10-01","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"* Added\n  * Switch the websocket client to the [Slipstream](https://github.com/NFIBrokerage/slipstream)\n    library for communication with NervesHub. There are no API changes. This should only be\n    a change to the internals, but you may notice timing differences especially around retries.\n\n## [0.12.1] - 2021-08-19\n\n* Added\n  * `NervesHubLink.reconnect/0` to force reconnection of the socket and channels","ref":"changelog.html#potentially-breaking","title":"Potentially Breaking - Changelog","type":"extras"},{"doc":"* fwup 1.0.0\n\n* Enhancements\n  * added `:fwup_env` option to the Configurator to support\n    environment variables which are needed in `fwup`","ref":"changelog.html#0-12-0","title":"0.12.0 - Changelog","type":"extras"},{"doc":"* Breaking Changes\n  * This release includes a change to how CA certificates are used in the connection\n    to NervesHub. If you are connecting to the publicly hosted https://nerves-hub.org,\n    then no changes are required.\n    If you are manually supplying `:ca_certs` config value to connect to another instance\n    of NervesHub, then you will need to update you config following the instructions of\n    [CA Certificates](https://hexdocs.pm/nerves_hub_link/readme.html#ca-certificates) in the README.","ref":"changelog.html#0-11-0","title":"0.11.0 - Changelog","type":"extras"},{"doc":"* Fixes\n  * Fix issue that allowed unresolved atom keys in `:fwup_public_keys`\n    config which would break firmware updates by failing to validate\n    the public key\n  * Add missing dependency on `:inets`","ref":"changelog.html#0-10-2","title":"0.10.2 - Changelog","type":"extras"},{"doc":"* Fixes\n  * Fix typo that would cause the device to fail to reboot after applying\n    an update","ref":"changelog.html#0-10-1","title":"0.10.1 - Changelog","type":"extras"},{"doc":"* `nerves_hub_link_common 0.2.0`\n\n* Potential Breaking Changes\n  * This enforces the update data structure exchanged between device and server\n    and is mostly internal. However, if you implement your own `NervesHubLink.Client`\n    behavior, then you will need to your `NervesHubLink.Client.update_available/1` to\n    accept a `%NervesHubLink.Message.UpdateInfo{}` struct as the parameter\n    instead of a map with string keys which was used until this point.\n\n* Enhancements\n  * Report API version to NervesHub. While there's currently only one version of\n    the Device API, this is anticipation that we may want to change it in the future.\n  * On reconnect, notify NervesHub of firmware currently being downloaded so\n    that NervesHub can differentiate failed firmware updates from network hiccups.\n  * Allow devices to implement their own reboot logic by implementing the\n    `NervesHubLink.Client.reboot/0` callback\n\n* Fixes\n  * Check `firmware_url` is valid before attempting update","ref":"changelog.html#0-10-0","title":"0.10.0 - Changelog","type":"extras"},{"doc":"* Enhancements\n  * Supports resuming failed downloads by offloading the responsibility of downloading\n  and applying updates to a new package: `nerves_hub_link_common`","ref":"changelog.html#0-9-4","title":"0.9.4 - Changelog","type":"extras"},{"doc":"* Fixes\n  * Fixes misleading default error message report\n  * Ensures `:nerves_key` is started before use to deal with optional dependency\n  start order bug","ref":"changelog.html#v0-9-3","title":"v0.9.3 - Changelog","type":"extras"},{"doc":"* Enhancements\n  * Supports incoming `window_size` message from web channel to change the TTY size","ref":"changelog.html#v0-9-2","title":"v0.9.2 - Changelog","type":"extras"},{"doc":"* Enhancements\n  * Send fwup version to server when connecting. This is required If you are using patchable firmware updates.","ref":"changelog.html#v0-9-1","title":"v0.9.1 - Changelog","type":"extras"},{"doc":"This release supports the new NervesHub console terminal. After upgrading,\nremote sessions should look almost the same as `ssh`-based ones: tab-completion,\ncolors, and commandline history work as expected now.","ref":"changelog.html#v0-9-0","title":"v0.9.0 - Changelog","type":"extras"},{"doc":"* Bug Fixes\n  * Fixes a broken call to `handle_fwup_message` on fwup success (thanks @bmteller! :heart:)","ref":"changelog.html#v0-8-2","title":"v0.8.2 - Changelog","type":"extras"},{"doc":"* Enhancements\n  * Flatten console data being sent to NervesHub\n  * Log all fwup progress percentage messages","ref":"changelog.html#v0-8-1","title":"v0.8.1 - Changelog","type":"extras"},{"doc":"* `nerves_hub_cli -> 0.10.0` - This decouples the CLI from the deprecated `:nerves_hub`\nlib and frees this lib to do the same. If you are setting `:nerves_hub, org: org` in\nyour config, compilation will fail until you change it to use this lib key:\n\n```elixir\nconfig :nerves_hub_link, org: org\n```\n* `nerves_hub_user_api -> 0.6.0`\n\n* Enhancements\n  * Updates example app\n  * Cleanup and structural changes\n  * The default NervesHub certificates are no longer stored in the `priv` directory so\n  if you're not using them, they won't be included.\n\n* Fixes\n  * Fixes an issue where a device may get an update message from the server while\n  performing an update which would cause things to crash.\n  * decouples `:nerves_hub` config values - see note above","ref":"changelog.html#v0-8-0","title":"v0.8.0 - Changelog","type":"extras"},{"doc":"Various cleanup and structure changes\n\n* Enhancements\n  * Add `Configurator` behavior - Gives the user a chance to do some configuration at runtime\n  * `:nerves_key` as optional dep","ref":"changelog.html#v0-7-6","title":"v0.7.6 - Changelog","type":"extras"},{"doc":"* Bug fixes\n  * Check NervesKey is provisioned before attempting to use it","ref":"changelog.html#v0-7-5","title":"v0.7.5 - Changelog","type":"extras"},{"doc":"* Enhancements\n  * Rename to `NervesHubDevice`\n  * Remove `HTTP` support to use channels exclusively\n  * Reorganize as a an `Application` and remove ability to start a supervisor separately\n  * Support using `NervesKey` via a configuartion flag","ref":"changelog.html#v0-7-4","title":"v0.7.4 - Changelog","type":"extras"},{"doc":"* Bug fixes\n  * Fix a NervesHub remote console issue due to missing `:io_request` handling\n\n* Enhancements\n  * Add support for querying the NervesHub connection state so that this is\n    easier to plug into Erlang's heart callback. This makes it possible to have\n    a device automatically reboot if it can't reach NervesHub for a long\n    time as a last ditch attempt at recovery. This has to be hooked up.","ref":"changelog.html#v0-7-3","title":"v0.7.3 - Changelog","type":"extras"},{"doc":"* Enhancements\n  * Report firmware update progress so it can be monitored on NervesHub.","ref":"changelog.html#v0-7-2","title":"v0.7.2 - Changelog","type":"extras"},{"doc":"* Bug fixes\n  * Handle firmware download connection timeout to fail more quickly when\n    connections fail","ref":"changelog.html#v0-7-1","title":"v0.7.1 - Changelog","type":"extras"},{"doc":"* New features\n  * Support remote IEx access from authorized NervesHub users, but only if\n    enabled. To enable, add `config :nerves_hub, remote_iex: true` to your\n    `config.exs`","ref":"changelog.html#v0-7-0","title":"v0.7.0 - Changelog","type":"extras"},{"doc":"* New features\n  * Support remote reboot request from NervesHub\n\n* Bug fixes\n  * Fix decoding of private key from KV","ref":"changelog.html#v0-6-0","title":"v0.6.0 - Changelog","type":"extras"},{"doc":"* Bug fixes\n  * Increased download hang timeout to deal with slow networks and <1 minute\n    long hiccups\n  * Fixed naming collision with a named process","ref":"changelog.html#v0-5-1","title":"v0.5.1 - Changelog","type":"extras"},{"doc":"* Enhancements\n  * nerves_hub_cli: Bump to v0.7.0\n  * The Phoenix Channel connection no longer uses the topic `firmware:firmware_uuid`\n    and instead connects to the topic `device`.","ref":"changelog.html#v0-5-0","title":"v0.5.0 - Changelog","type":"extras"},{"doc":"This release has backwards incompatible changes so please read carefully.\n\nThe configuration key for firmware signing keys has changed from `:public_keys`\nto `:fwup_public_keys`.\n\nIf you are not using nerves-hub.org for your NervesHub server, the configuration\nkeys for specifying the device endpoint for the server have changed. Look for\n`:device_api_host` and `:device_api_port` in the documentation and example for\nsetting these.\n\n* Enhancements\n  * All firmware metadata is now passed up to the NervesHub. This will make it\n    possible for the server to make decisions on firmware that has been loaded\n    outside of NervesHub or old firmware that has been unloaded from NervesHub.\n  * Code cleanup and refactoring throughout. More passes are planned.","ref":"changelog.html#v0-4-0","title":"v0.4.0 - Changelog","type":"extras"},{"doc":"* Enhancements\n  * Add uuid and dn to http headers for polling requests\n\n* Bug fixes\n  * Fix crash when no updates were available","ref":"changelog.html#v0-3-0","title":"v0.3.0 - Changelog","type":"extras"},{"doc":"* Bug fixes\n  * Use CA certificates from `:nerves_hub` instead of `:nerves_hub_core`.","ref":"changelog.html#v0-2-1","title":"v0.2.1 - Changelog","type":"extras"},{"doc":"* Enhancements\n  * Updated docs.\n  * Added support for [NervesKey](https://github.com/nerves-hub/nerves_key).\n  * Added support for performing conditional updates.\n  * Include `fwup` elixir dependency for interfacing with `fwup`.\n  * Update deps and code to make it possible to run on host for testing.\n  * Automatically call `NervesHub.connect()` instead of requiring it to be specified.\n  * Improved error handling and reporting.","ref":"changelog.html#v0-2-0","title":"v0.2.0 - Changelog","type":"extras"},{"doc":"Initial release","ref":"changelog.html#v0-1-0","title":"v0.1.0 - Changelog","type":"extras"},{"doc":"# Configuration","ref":"configuration.html","title":"Configuration","type":"extras"},{"doc":"`NervesHubLink` also supports runtime configuration via the `NervesHubLink.Configurator` behavior. This is called during application startup to build the configuration that is to be used for the connection. When implementing the behavior, you'll receive the initial default config read in from the application environment and you can modify it however you need.\n\nThis is useful for cases like:\n\n- selectively choosing which cert/key to use\n- reading a certificate file stored on the device which isn't available during compilation\n\nFor example:\n\n```elixir\ndefmodule MyApp.Configurator do\n  @behaviour NervesHubLink.Configurator\n\n  @impl NervesHubLink.Configurator\n  def build(config) do\n    ssl = [certfile: \"/root/ssl/cert.pem\", keyfile: \"/root/ssl/key.pem\"]\n    %{config | ssl: ssl}\n  end\nend\n```\n\nThen you specify which configurator `NervesHubLink` should use in `config.exs`:\n\n```elixir\nconfig :nerves_hub_link, configurator: MyApp.Configurator\n```","ref":"configuration.html#runtime-configuration","title":"Runtime configuration - Configuration","type":"extras"},{"doc":"Firmware and Archive downloads are resilient to network issues and can be retried automatically.\n\nYou can configure how the Downloader handles timeouts, disconnections, and other aspects of the\nretry logic by adding the following configuration to your application's config file:\n\n```elixir\nconfig :nerves_hub_link, :retry_config,\n  max_disconnects: 20,\n  idle_timeout: 75_000,\n  max_timeout: 10_800_000\n```\n\nFor more information about the configuration options, see the `NervesHubLink.Downloader.RetryConfig` module.","ref":"configuration.html#retrying-firmware-downloads","title":"Retrying firmware downloads - Configuration","type":"extras"},{"doc":"It's not always appropriate to apply a firmware update immediately. Custom logic can be added to the device by\nimplementing the `NervesHubLink.Client` behaviour and telling NervesHubLink to use it. eg.\n\n```elixir\ndefmodule MyApp.NervesHubLinkClient do\n   @behaviour NervesHubLink.Client\n\n   # May return:\n   #  * `:apply` - apply the action immediately\n   #  * `:ignore` - don't apply the action, don't ask again.\n   #  * `{:reschedule, timeout_in_milliseconds}` - call this function again later.\n\n   @impl NervesHubLink.Client\n   def update_available(data) do\n    if SomeInternalAPI.is_now_a_good_time_to_update?(data) do\n      :apply\n    else\n      {:reschedule, 60_000}\n    end\n   end\nend\n```\n\nTo have NervesHubLink use it, update your `config.exs` as follows:\n\n```elixir\nconfig :nerves_hub_link, client: MyApp.NervesHubLinkClient\n```","ref":"configuration.html#conditionally-applying-updates","title":"Conditionally applying updates - Configuration","type":"extras"},{"doc":"See the previous section for implementing a `client` behaviour.\n\n```elixir\ndefmodule MyApp.NervesHubLinkClient do\n  @behaviour NervesHubLink.Client\n  #  argument can be:\n  #   {:ok, non_neg_integer(), String.t()}\n  #   {:warning, non_neg_integer(), String.t()}\n  #   {:error, non_neg_integer(), String.t()}\n  #   {:progress, 0..100}\n  def handle_fwup_message({:ok, _, _}) do\n    Logger.error(\"Firmware update complete\")\n    :ok\n  end\n\n  def handle_fwup_message({:warning, code, message}) do\n    Logger.error(\"Warning while applying firmware update (#{code)}): #{message}\")\n    :ok\n  end\n\n  def handle_fwup_message({:error, _, message}) do\n    Logger.error(\"Error while applying firmware update #(#{code}): {message}\")\n    :ok\n  end\n\n  def handle_fwup_message({:progress, progress}) when rem(progress, 10) do\n    Logger.info(\"Update progress: #{progress}%\")\n    :ok\n  end\n\n  def handle_fwup_message({:progress, _}) do\n    :ok\n  end\nend\n```","ref":"configuration.html#reporting-update-progress","title":"Reporting update progress - Configuration","type":"extras"},{"doc":"It's possible to remotely log into your device via the NervesHub web interface. This feature is disabled by default. To enable, add the following to your `config.exs`:\n\n```elixir\nconfig :nerves_hub_link, remote_iex: true\n```\n\nThe remote IEx process is started on the first data request from NervesHub and is terminated after 5 minutes of inactivity. You can adjust this by setting `:remote_iex_timeout` value in seconds in your `config.exs`:\n\n```elixir\nconfig :nerves_hub_link, remote_iex_timeout: 900 # 15 minutes\n```\n\nYou may also need additional permissions on NervesHub to see the device and to use the remote IEx feature.","ref":"configuration.html#enabling-remote-iex-access","title":"Enabling remote IEx access - Configuration","type":"extras"},{"doc":"This application can set and clear the following alarms:\n\n- `NervesHubLink.Disconnected`\n  - set: An issue is preventing a connection to NervesHub or one just hasn't been made yet\n  - clear: Currently connected to NervesHub\n- `NervesHubLink.UpdateInProgress`\n  - set: A new firmware update is being downloaded or applied\n  - clear: No updates are happening","ref":"configuration.html#alarms","title":"Alarms - Configuration","type":"extras"},{"doc":"The CA certificates installed on the device are used by default.\n\nIf you include the [CAStore](https://hex.pm/packages/castore) in your project, then that will be selected and used.\n\nOtherwise you can configure `nerves_hub_link` to use custom CA certificates, which is useful if you are running your own NervesHub instance with self signed SSL certificates. Use the `:ca_store` option to specify a module with a `ca_certs/0` function that returns a list of DER encoded certificates:\n\n```elixir\nconfig :nerves_hub_link, ca_store: MyModule\n```\n\nOr if you have the certificates in DER format, you can also explicitly set them in the :ssl option:\n\n```elixir\nmy_der_list = [<<213, 34, 234, 53, 83, 8, 2, ...>>]\nconfig :nerves_hub_link, ssl: [cacerts: my_der_list]\n```","ref":"configuration.html#ca-certificates","title":"CA Certificates - Configuration","type":"extras"},{"doc":"`NervesHubLink` will attempt to verify that the network is available before initiating the first connection attempt. This is done by checking if the `NervesHub` host address (`config.host`) can be resolved. If the network isn't available then the check will be run again in 2 seconds.\n\nYou can disable this behaviour with the following config:\n\n```elixir\nconfig :nerves_hub_link, connect_wait_for_network: false\n```","ref":"configuration.html#verifying-network-availability","title":"Verifying network availability - Configuration","type":"extras"},{"doc":"To disable `NervesHubLink` connecting to `NervesHub` when testing, you can add:\n\n```elixir\nconfig :nerves_hub_link, connect: false\n```\n\nto your `config/test.exs`","ref":"configuration.html#disable-nerveshublink-during-testing","title":"Disable `NervesHubLink` during testing - Configuration","type":"extras"},{"doc":"# Extensions: Health and Geo\n\nExtensions are pieces of non-critical functionality going over the NervesHub WebSocket. They are separated out under the Extensions mechanism so that the client can happily ignore anything extension-related in service of keeping firmware updates healthy. That is always the top priority.\n\nThere are two extensions currently:\n\n- **Health** reports device metrics, alarms, metadata and similar.\n- **Geo** provides GeoIP information and allows slotting in a better source.\n\nYour NervesHub server controls enabling and disabling extensions to allow you to switch them off if they impact operations.","ref":"extensions.html","title":"Extensions","type":"extras"},{"doc":"You can add your own metrics, metadata and alarms.\n\nThe default set of metrics used by the `Health.DefaultReport` are:\n\n- `NervesHubLink.Extensions.Health.MetricSet.CPU` - CPU temperature, usage (percentage), and load averages.\n- `NervesHubLink.Extensions.Health.MetricSet.Memory` - Memory size (MB), used (MB), and percentage used.\n- `NervesHubLink.Extensions.Health.MetricSet.Disk` - Disk size (KB), available (KB), and percentage used.\n\nAnd one optional metric set:\n- `NervesHubLink.Extensions.Health.MetricSet.NetworkTraffic` - Total bytes sent and received (per interface).\n\nYou can also create your own metric sets by implementing the `NervesHubLink.Extensions.Health.MetricSet`\nbehaviour.\n\nIf a library you are using provides a metric set, you can add it to the list of metrics, but please ensure\nto include all the metric sets you want to use. If you want to include the full default set, you can use\n`:default` or `:defaults` in your metric set list.\n\neg.\n\n```\nconfig :nerves_hub_link,\n  health: [\n    metric_sets: [\n      :defaults,\n      MyApp.HealthMetrics,\n      ALibrary.BatteryMetrics\n    ]\n  ]\n```\n\nIf you only want to use some of the default metrics, you can specify them explicitly:\n\n```\nconfig :nerves_hub_link,\n  health: [\n    metric_sets: [\n      NervesHubLink.Extensions.Health.MetricSet.CPU,\n      NervesHubLink.Extensions.Health.MetricSet.Memory\n      # the disk metrics have been excluded\n    ]\n  ]\n```\n\nAnd if you don't want to use any metric sets, you can set the `metric_sets` option to an empty list.\n\n```\nconfig :nerves_hub_link,\n  health: [\n    metric_sets: []\n  ]\n```\n\nIf you want to add custom metadata to the default health report, you can specify it with:\n\n```\nconfig :nerves_hub_link,\n  health: [\n    # metadata is added with a key and MFA\n    # the function should return a string\n    metadata: %{\n      \"placement\" => {CatCounter, :venue, []}\n    }\n  ]\n```\n\nOr you can implement a completely custom reporting module by implementing `NervesHubLink.Extensions.Health.Report` and configuring it:\n\n```\nconfig :nerves_hub_link,\n  health: [\n    report: CatCounter.MyHealthReport\n  ]\n```","ref":"extensions.html#health","title":"Health - Extensions","type":"extras"},{"doc":"It is intended to be easy to replace the default Geo Resolver with your own. Maybe you have a GPS module or can resolve a reasonably precise location via LTE. Just change config:\n\n```\nconfig :nerves_hub_link,\n  geo: [\n    resolver: CatCounter.MyResolver\n  ]\n```\n\nYour module only needs to implement a single function, see `NervesHubLink.Extensions.Geo.Resolver` for details.","ref":"extensions.html#geolocation","title":"Geolocation - Extensions","type":"extras"},{"doc":"The [default health report](`NervesHubLink.Extensions.Health.DefaultReport`) requires the use of\nthe [`alarmist`](https://hex.pm/packages/alarmist) library for correct alarms handling.\n\nTo use another handler, you will need to create a custom [health report](`NervesHubLink.Extensions.Health.Report`)\nwhich implements the [`alarms/0`](`c:NervesHubLink.Extensions.Health.Report.alarms/0`) callback.","ref":"extensions.html#alarms","title":"Alarms - Extensions","type":"extras"},{"doc":"# Debugging","ref":"debugging.html","title":"Debugging","type":"extras"},{"doc":"If you see the following in your logs:\n\n```text\n14:26:06.926 [info]  ['TLS', 32, 'client', 58, 32, 73, 110, 32, 115, 116, 97, 116, 101, 32, 'cipher', 32, 'received SERVER ALERT: Fatal - Unknown CA', 10]\n```\n\nThis probably indicates that the signing certificate hasn't been uploaded to NervesHub so the device can't be authenticated. Double check that you ran:\n\n```sh\nmix nerves_hub.ca_certificate register my-signer.cert\n```\n\nAnother possibility is that the device wasn't provisioned with the certificate\nthat's on NervesHub.\n\nSee also [NervesHubWeb: Potential SSL Issues](https://github.com/nerves-hub/nerves_hub_web#potential-ssl-issues)","ref":"debugging.html#tls-client-errors","title":"TLS client errors - Debugging","type":"extras"}],"producer":{"name":"ex_doc","version":"0.37.3"},"proglang":"elixir"}